program ising     !calcularemos las curvas de las propiedades medias alrededor de la zona critica
                  !usando el algoritmo de metropolis

 integer, parameter:: L=100       !numero de spines en cada una de las dos direcciones
 real:: temperatura
 integer, parameter:: n_spines=L*L       !numero total de spines
 integer, dimension (n_spines):: s      !el indice del vector ira de 1 hasta L**2. Es el vector que contiene todos los spines
 integer, dimension (n_spines,4):: lista_vecinos         !guardamos los vecinos correspondientes a cada spin
 integer, parameter:: iteraciones_termalizacion_1=25000000, iteraciones_media=100000000,& 
      iteraciones_termalizacion_2=75000000, n0_datos=100000  !primero dejamos que termalice, y luego hacemos mas iteraciones
                                                             !cogiendo datos cada n0_datos iteraciones para las medias          
 integer:: i
  
 !definimos lo que necesitaremos para el proceso en un rango de temperaturas
 real, parameter:: salto_T_1=0.05, salto_T_2=0.01    !cogemos 0.05 entre 1.5 y 2.15, y 0.01 entre 2.15 y 2.35, y 0.05 de 2.35 a 3.0
                                               !(si cogemos otro salto habra que revisar 
                                               !el do correspondiente, porque habra que hacer un numero diferente de iteraciones)
 integer:: iter_T         !el iterador para las temperaturas
 
 
 open(unit=2, file="curvas.txt", status="replace", action="write")      !aqui guardaremos las medias para hacer las curvas de las propiedades

 !lo primero es conseguir una distribucion de spines inicial, con todos los spines ordenados
 
 s=1
 
 !definimos una lista de vectores de 4 elementos, para tener los vecinos de cada spin
 
 do i= 1, n_spines
 
  lista_vecinos(i, :)= prox_vecinos(i,L)
 
 end do
 
 
 
 !ahora hay que hacer el algoritmo de Metropolis para cada temperatura
 !simplemente cogemos un spin al azar y lo cambiamos de posicion (multiplicamos por -1)
 !entonces, vemos la diferencia de energia respecto a la configuracion anterior y aplicamos las normas de transicion de Metropolis
     
 do iter_T=0,13                !la temperatura ira desde 1.5 a 2.15 con un salto de 0.05
  
  temperatura=1.5 + salto_T_1*real(iter_T)
  
  call algoritmo(s, temperatura, iteraciones_termalizacion_1, iteraciones_media, n0_datos, lista_vecinos, n_spines)
  
 end do
 
 do iter_T=1,25                !la temperatura ira desde 2.15 a 2.40 con un salto de 0.01
  
  temperatura=temperatura + salto_T_2
  
  call algoritmo(s, temperatura, iteraciones_termalizacion_2, iteraciones_media, n0_datos, lista_vecinos, n_spines)
  
 end do

 do iter_T=1,12                !la temperatura ira desde 2.40 a 3.0 con un salto de 0.05 de nuevo
  
  temperatura=temperatura + salto_T_1
  
  call algoritmo(s, temperatura, iteraciones_termalizacion_1, iteraciones_media, n0_datos, lista_vecinos, n_spines)
  
 end do
 
 close(unit=2)
 
 print *, "El proceso ha finalizado"
 
 
 
 
 
!--------------------------------------------------------------------------------------------------------------------------- 
contains
 
 function prox_vecinos(sitio, L) result (vecinos)      !meto un entero indicando el elemento de s cuyos vecinos quiero
                                                       !me devuelve las posiciones de los vecinos en s (es decir, la posicion "unidimensional")
  integer, intent(in):: sitio, L
  integer, dimension(4):: vecinos         !vecinos(1) es el de arriba, el 2 el de abajo, el 3 el de la izquierda, el 4 el de derecha
  integer, dimension(4):: vecinos_aux
  
  if (sitio - L<=0) then       !esta en el lado superior del cuadrado, impongo condiciones periodicas de contorno   
   vecinos_aux(1)=L*L - L + sitio  
  else   
   vecinos_aux(1)=sitio - L   
  end if
  
  if (sitio + L > L*L) then           !esta en el lado inferior   
   vecinos_aux(2)=L - L*L + sitio  
  else   
   vecinos_aux(2)=sitio + L  
  end if
    
  if (sitio - 1==0 .or. modulo(sitio - 1, L)==0) then          !esta en el lado izquierdo   
   vecinos_aux(3)=sitio + L - 1  
  else   
   vecinos_aux(3)=sitio - 1  
  end if
  
  if (modulo(sitio, L)==0) then                    !esta en el lado derecho   
   vecinos_aux(4)=sitio + 1 - L  
  else   
   vecinos_aux(4)=sitio + 1  
  end if 
  
  vecinos=vecinos_aux    
 end function prox_vecinos 
 
 subroutine random_position(pos, n_spines)         !cada vez que llamemos a esta subrutina obtendremos la posicion de un spin aleatoriamente
  integer, intent(in):: n_spines
  integer, intent(inout):: pos
  real:: aux
  real:: d         !otra auxiliar
  
  call random_seed()
  call random_number(aux)
  
  d=aux*real(n_spines) + 1.0    
  pos=int(d)                    !asi, obtenemos la misma probabilidad para los valores desde 1 hasta L**2  
 end subroutine random_position
 
 function calculo_energia(spines, lista_vecinos, n_spines) result (energia)        !obtiene la energia de la configuracion dada por el vector "spines"
  
  integer, intent(in):: n_spines
  integer, dimension (:), intent(in):: spines
  integer, dimension(:,:), intent (in):: lista_vecinos
  real:: energia, energia_aux
  integer, dimension (4):: vec
  integer:: posicion
  integer:: i       !para recorrer todos los spines
  
  energia_aux=0.0
  
  do i=1,n_spines   
   vec=lista_vecinos(i, :)
   energia_aux=energia_aux - real(spines(i))*real(spines(vec(1))+spines(vec(2))+spines(vec(3))+spines(vec(4)))   
  end do
  
  energia=energia_aux/2.0     !para no contar cada pareja dos veces  
 end function calculo_energia
 
 subroutine algoritmo(s, temperatura, iteraciones_termalizacion, iteraciones_media, n0_datos, lista_vecinos, n_spines)
  
  integer, dimension(:), intent(inout):: s     !va variando a lo largo del proceso
  real, intent (in):: temperatura
  integer, intent(in):: iteraciones_termalizacion, iteraciones_media, n0_datos
  integer, dimension(:,:), intent(in):: lista_vecinos
  integer, intent(in):: n_spines
  real, dimension(4):: exponencial         !guardamos los posibles valores de exp(-delta_E/temperatura), en cada temperatura
  real:: prob_transicion, r
  integer, dimension(4):: vecinos          !para guardar las posiciones de los cuatro vecinos de un spin
  integer:: delta_E
  integer:: posicion            !para coger un spin aleatoriamente, usando la subrutina del contains
  integer:: i
    
  !definimos variables para acumular las sumas para hacer la media de las propiedades
  real:: suscept, calor, energia_instantanea, suma_aux  
  real:: energia, energia_2, magnetizacion, magnetizacion_2
  real:: energia_media, energia_2_media, magnetizacion_media, magnetizacion_2_media
  character(len=*), parameter:: formato="(f4.2,tr1,f4.2,tr1,f5.2,tr1,f8.3,tr1,f7.3)"        !para escribir en el fichero
      
  !delta_E puede tomar solo como valores: 8,6,4,2,0,-2,-4,-6 y -8, o sea, 9 valores diferentes. Solo calculamos la exponencial
  !si la delta_E es >0, por lo que tenemos unicamente 4 valores diferentes para la exponencial
   
  do i=1,4   
   exponencial(i)=exp(-2.0*real(i)/temperatura)   
  end do
    
  do i=1,iteraciones_termalizacion  
   call random_position(posicion, n_spines)      !generamos una posicion aleatoria de un spin, el cual vamos a voltear
   
   !calculamos la variacion de energia correspondiente al volteo, Ef - Ei (como la energia es menos la suma de los pares de spines, podemos
   !expresar delta_E como: suma de los pares de la configuracion inicial - la suma de los pares de la final)
   !ademas, la diferencia tendra que ver unicamente con el spin volteado (s(pos)=>-s(pos))
   
   vecinos=lista_vecinos(posicion,:)   
   delta_E=s(posicion) * 2 * (s(vecinos(1)) + s(vecinos(2)) + s(vecinos(3)) + s(vecinos(4)))
   
   !ahora que tenemos la variacion de la energia, debemos ver
   !si efectivamente cambiamos la configuracion o no, mediante las reglas de transicion 
   
   if (delta_E <= 0) then        !en este caso se acepta la transicion   
    s(posicion)=-s(posicion)    !volteamos el spin considerado   
   else                !en este caso, se acepta con una cierta probabilidad      
    call random_seed()
    call random_number(r)        !generamos un numero aleatorio entre 0 y 1    
    prob_transicion= exponencial(delta_E/2)
    
    if (r<prob_transicion) then       !aceptamos la transicion. Si no, no la aceptamos y pasamos a la siguiente iteracion     
     s(posicion)=-s(posicion)             
    end if       
   end if
      
   if (modulo(i,10000000)==0) then                      !para controlar la ejecucion del programa   
    print *, "Quedan ", iteraciones_termalizacion - i, " iteraciones del proceso de termalizacion para T=", temperatura   
   end if    
  end do
 
  !ahora que ya tenemos el sistema termalizado podemos ir cogiendo datos para hacer las medias
    
  energia=0.0        !inicializamos las variables
  energia_2=0.0
  magnetizacion=0.0
  magnetizacion_2=0.0
  
  do i=1,iteraciones_media  
   call random_position(posicion, n_spines)      !generamos una posicion aleatoria de un spin, el cual vamos a voltear   
   vecinos=lista_vecinos(posicion, :)        !obtenemos los 4 vecinos   
   delta_E=s(posicion) * 2 * (s(vecinos(1)) + s(vecinos(2)) + s(vecinos(3)) + s(vecinos(4)))
      
   if (delta_E <= 0) then       !en este caso se acepta la transicion   
    s(posicion)=-s(posicion)    !volteamos el spin considerado   
   else                         !en este caso, se acepta con una cierta probabilidad      
    call random_seed()
    call random_number(r)       !generamos un numero aleatorio entre 0 y 1    
    prob_transicion= exponencial(delta_E/2)
    
    if (r<prob_transicion) then       !aceptamos la transicion. Si no, no la aceptamos y pasamos a la siguiente iteracion     
     s(posicion)=-s(posicion)             
    end if       
   end if
      
   if (modulo(i,n0_datos)==0) then          !condicion para tomar datos para la media    
    energia_instantanea= calculo_energia(s, lista_vecinos, n_spines)     !calculamos la energia del sistema mediante la funcion del contains
    suma_aux=real(sum(s))
    
    magnetizacion=magnetizacion +  abs(suma_aux)
    magnetizacion_2=magnetizacion_2 + (suma_aux)**2.0
    energia=energia + energia_instantanea
    energia_2=energia_2 + (energia_instantanea)**2.0    
   end if
      
   if (modulo(i,10000000)==0) then                      !para controlar la ejecucion del programa   
    print *, "Quedan ", iteraciones_media - i, " iteraciones del proceso de medias para la T=", temperatura   
   end if   
  end do
    
  magnetizacion_media=magnetizacion*real(n0_datos)/real(iteraciones_media)
  magnetizacion_2_media=magnetizacion_2*real(n0_datos)/real(iteraciones_media)
  energia_media=energia*real(n0_datos)/real(iteraciones_media)
  energia_2_media=energia_2*real(n0_datos)/real(iteraciones_media)
  
  suscept=(magnetizacion_2_media - magnetizacion_media**2.0)/temperatura
  calor=(energia_2_media - energia_media**2.0)/(temperatura**2.0)
   
  !escribimos las medias en el fichero (escribimos la propiedad por spin, por lo que dividimos cada cantidad por el numero de spines total) 
    
  write(unit=2, fmt=formato) temperatura, magnetizacion_media/real(n_spines), &
                       energia_media/real(n_spines), suscept/real(n_spines), calor/real(n_spines) 
 end subroutine algoritmo
end program ising