program grafos_aleatorios    !observaremos hacia donde evoluciona el sistema
                             !usaremos el modelo de Sznajd A    

 integer, parameter:: n_spines=500           !numero total de spines, el cual debemos modificar para cada grafo considerado
 real:: magn_deseada                         !magnetizacion (negativa) que queremos en la c. inicial
 integer, dimension (n_spines,n_spines - 1):: lista_vecinos         !guardamos los vecinos correspondientes a cada spin 
                                                                    !(como mucho, un spin puede tener n_spines-1 vecinos)
 integer:: numero_iostat              !para controlar la lectura de los datos del fichero "grafo_500.txt"
 integer, parameter:: iteraciones=2500000     !como mucho estas iteraciones (5000 MCS)          
 integer:: i,j
 
 open(unit=2, file="curva500.txt", status="replace", action="write")
 
 !definimos la lista de vecinos de cada spin, que tenemos guardada en un fichero

 open(unit=1, file="grafo_500.txt", status="old", action="read") 
 lista_vecinos=0     !inicializamos a 0. Asi, cuando lleguemos a un 0 significara que no tiene mas vecinos el spin considerado
 
 principal: do i=1,n_spines  
  do j=1,n_spines-1   
   read (unit=1, fmt=*, iostat=numero_iostat) lista_vecinos(i,j)

   if (numero_iostat>0) then            !ha intentado leer un dato de tipo character, una T, que
                                        !hemos puesto para indicar que el spin actual no tiene mas vecinos    
    exit    !pasamos a los vecinos del siguiente spin    
   else if (numero_iostat<0) then        !hemos llegado al final del fichero de datos, hemos terminado la lista de vecinos   
    exit principal    
   end if
  end do
 end do principal
 close(unit=1)
  
 do i=0,6
  magn_deseada=-0.5 + 0.05 * real(i)
  print *, "Magnetizacion inicial es: ", magn_deseada
  call algoritmo(magn_deseada, n_spines, iteraciones, lista_vecinos)  
 end do
 
 do i=1,10                !hasta 0 cada 0.02, para analizar mejor este rango
  magn_deseada=-0.2 + 0.02 * real(i)
  print *, "Magnetizacion inicial es: ", magn_deseada
  call algoritmo(magn_deseada, n_spines, iteraciones, lista_vecinos)
 end do
 
 close(unit=2)
  
 
!--------------------------------------------------------------------------------------------------------------------------- 
contains
 
 subroutine random_position(pos, n_spines)         !cada vez que llamemos a esta subrutina obtendremos la posicion de un spin aleatoriamente
  integer, intent(in):: n_spines                       !metemos el numero total de spines
  integer, intent(inout):: pos
  real:: aux
  real:: d         !otra auxiliar
  
  call random_seed()
  call random_number(aux)
  
  d=aux*real(n_spines) + 1.0    
  pos=int(d)            !asi, obtenemos la misma probabilidad para los valores desde 1 hasta n_spines  
 end subroutine random_position
 
 subroutine conf_inicial(spines, n_spines, magn_inicial)      !magn_inicial es la que queremos que haya en la configuracion inicial  
  real, intent (in):: magn_inicial
  integer, intent (in):: n_spines
  integer, dimension(:), intent (inout):: spines
  integer:: posicion, contador
  integer:: n_sies
  
  n_sies=nint((1.0 + magn_inicial)*real(n_spines)/2.0)      !obtenemos el numero de spines up que necesitamos para tener esa magn_inicial
  print *, "Numero de sies, para ver si es corrrecto: ", n_sies
  
  spines=0
  contador=0     !lo usaremos para saber a cuantos spines hemos asignado un 1, parando el loop cuando contador=n_sies
  
  do   
   if (contador==n_sies) then      !ya hemos asignado la cantidad que queriamos
    exit
   else if (contador>n_spines) then       !solo podemos asignar valores como mucho a n_spines spines
    print *, "Ha habido un error en la subrutina conf_inicial"
    stop 
   end if
   
   call random_position(posicion, n_spines)
   if (spines(posicion)==1) then         !ya hemos asignado un valor a ese spin
    cycle
   end if
   
   spines(posicion)=1
   contador=contador + 1   
  end do
  
  where (spines==0)    !asignamos -1 al resto de spines
   spines=-1
  end where  
 end subroutine conf_inicial
  
 subroutine algoritmo(magn_deseada, n_spines, iteraciones_max, lista_vecinos)
  
  integer, intent (in):: n_spines
  real, intent (in):: magn_deseada
  integer, dimension (n_spines):: s, s_copia      !el indice del vector ira de 1 hasta n_spines. Es el vector que contiene todos los spines
                                                 !en la copia guardaremos una configuracion inicial, la que usaremos en todas las simulaciones
  integer, dimension (:,:), intent (in):: lista_vecinos         !aqui estan los vecinos correspondientes a cada spin
  integer, intent (in):: iteraciones_max          !como mucho estas iteraciones para termalizar el sistema          
  integer:: i,j,t           !para hacer el loop de los procesos de montecarlo y el de los vecinos
  integer:: posicion1, posicion2             !para coger spines aleatoriamente, usando la subrutina del contains
  integer, dimension(n_spines-1):: vecinos1, vecinos2         !para guardar las posiciones de los cuatro vecinos de cada uno de los dos spines
  real:: magn       !para ir calculando la magnetizacion actual
  integer:: aux         !a la hora de coger un vecino al azar de un spin, uso call random_position(aux, 4). Esto me devuelve en aux
                        !la posicion en vecinos1 que debo coger para la posicion en el vector de spines del spin vecino
  
  integer::numero_vecinos 
  integer:: iterador, simulacion, contador_sies, contador_noes, contador_excepciones
  
  !para hacer los errores y medias
  real, dimension(1:10):: magn_media           !calcularemos la magnetizacion de la situacion final promediada a todas las simulaciones 
  real:: valor_medio, error
   
  do iterador=1,10       !para conseguir 10 puntos para hacer las medias y error  
   !lo primero es conseguir la distribucion de spines inicial, con la subrutina del contains
   call conf_inicial(s_copia,n_spines,magn_deseada)
       
   !ahora hay que hacer el algoritmo
   !simplemente cogemos un spin al azar y uno de sus vecinos
   !entonces, si opinan igual convencen a todos sus vecinos, si opinan diferente no ocurre nada
   !vamos a hacer varias simulaciones, y vamos a ver la media de la magnetizacion final, contando el n de veces que se llega a cada situacion
   
   contador_excepciones=0
   contador_sies=0
   contador_noes=0
   
   do simulacion=1,1000        
    if (modulo(simulacion, 250)==0) then
     print *, "Va por la simulacion ", simulacion, " de la iteracion ", iterador
    end if
    
    s=s_copia                !inicializamos a la conf inicial que tenemos guardada
       
    do i=1,iteraciones     
     call random_position(posicion1, n_spines)      !generamos una posicion aleatoria de un spin
     vecinos1=lista_vecinos(posicion1,:)     !guardamos sus vecinos
     
     !elegimos uno de sus vecinos al azar, debiendo primero saber cuantos vecinos tiene   
     numero_vecinos=0
     do j=1, n_spines - 1  
      if (vecinos1(j)==0) then
       exit
      end if
      numero_vecinos=numero_vecinos+1
     end do
     
     if (numero_vecinos==0) then
      cycle
     end if
     
     call random_position(aux, numero_vecinos)
     posicion2=vecinos1(aux)
     
     if (s(posicion1)*s(posicion2)==1) then       !tienen la misma opinion, por lo que se la imponen a sus vecinos. Si no, no hacemos nada      
      do t=1, numero_vecinos     
       s(vecinos1(t))=s(posicion1)       
      end do
      
      vecinos2=lista_vecinos(posicion2,:)         !hacemos el mismo procedimiento para el spin 2
      numero_vecinos=0           
      do j=1, n_spines - 1
       if (vecinos2(j)==0) then
        exit
       end if
       numero_vecinos=numero_vecinos+1
      end do
      
      do t=1,numero_vecinos       
       s(vecinos2(t))=s(posicion1)       
      end do          
     end if 
     
     if (modulo(i,100)==0) then    !comprobamos si se ha alcanzado una de las dos situaciones finales cada 100 iteraciones      
      magn=real(sum(s))/real(n_spines)
      
      if (magn==1.0 .or. magn==-1.0) then                  
       if (magn==1.0) then
        contador_sies=contador_sies + 1
       else 
        contador_noes=contador_noes + 1 
       end if
       
       exit       
      end if      
     end if
     
     if (i==iteraciones) then      
      contador_excepciones=contador_excepciones + 1      
     end if     
    end do   
   end do
   
   magn_media(iterador)=real(contador_sies - contador_noes)/real(contador_sies + contador_noes)    
  end do
  
  valor_medio=sum(magn_media)/10.0
  
  error=0.0
  do i=1,10
   error=error + (magn_media(i) - valor_medio)**2.0
  end do
  error=sqrt(error/(10.0*9.0))
  
  write(unit=2, fmt="(f5.2,tr1,f7.4,tr1,f7.5)") magn_deseada, valor_medio, error   
 end subroutine algoritmo  
end program grafos_aleatorios