program sznajd              !observaremos, con diferentes condiciones iniciales, hacia donde evoluciona el sistema
                            !usaremos el modelo de Sznajd A    

 integer, parameter:: L=50                    !numero de spines en cada una de las dos direcciones
 integer, parameter:: n_spines=L*L            !numero total de spines
 real:: magn_deseada                          !magnetizacin que queremos en la configuracion inicial
 integer, parameter:: iteraciones=2500000     !como mucho estas iteraciones (1000 MCS) para termalizar el sistema
 integer:: i
 integer, dimension (n_spines,4):: lista_vecinos         !guardamos los vecinos correspondientes a cada spin
 
 open(unit=1, file="curva.txt", status="replace", action="write")

 !definimos una lista de vectores de 4 elementos, para tener los vecinos de cada spin
 do i= 1, n_spines
  lista_vecinos(i, :)= prox_vecinos(i,L)
 end do
 
 !realizamos para un rango de magnetizaciones iniciales el algoritmo de Sznajd
  
 do i=0,8
  magn_deseada=-1.0 + 0.1 * real(i)
  print *, "Magnetizacion inicial es: ", magn_deseada
  call algoritmo(magn_deseada, n_spines, iteraciones, lista_vecinos)
 end do
 
 do i=1,20                !hasta +0.2 cada 0.02, para analizar mejor este rango
  magn_deseada=-0.2 + 0.02 * real(i)
  print *, "Magnetizacion inicial es: ", magn_deseada
  call algoritmo(magn_deseada, n_spines, iteraciones, lista_vecinos)
 end do
 
 do i=1,8
  magn_deseada=0.2 + 0.1 * real(i)
  print *, "Magnetizacion inicial es: ", magn_deseada
  call algoritmo(magn_deseada, n_spines, iteraciones, lista_vecinos)
 end do
 
 close(unit=1)
 
!--------------------------------------------------------------------------------------------------------------------------- 
contains
 
 function prox_vecinos(sitio, L) result (vecinos)      !meto un entero indicando el elemento de s cuyos vecinos quiero
                                                       !me devuelve las posiciones de los vecinos en s (es decir, la posicion "unidimensional")  
  integer, intent(in):: sitio, L
  integer, dimension(4):: vecinos         !vecinos(1) es el de arriba, el 2 el de abajo, el 3 el de la izquierda, el 4 el de derecha
  integer, dimension(4):: vecinos_aux
  
  if (sitio - L<=0) then       !esta en el lado superior del cuadrado, impongo condiciones periodicas de contorno   
   vecinos_aux(1)=L*L - L + sitio  
  else   
   vecinos_aux(1)=sitio - L   
  end if
  
  if (sitio + L > L*L) then           !esta en el lado inferior   
   vecinos_aux(2)=L - L*L + sitio  
  else   
   vecinos_aux(2)=sitio + L   
  end if
  
  if (sitio - 1==0 .or. modulo(sitio - 1, L)==0) then          !esta en el lado izquierdo   
   vecinos_aux(3)=sitio + L - 1  
  else  
   vecinos_aux(3)=sitio - 1  
  end if
  
  if (modulo(sitio, L)==0) then                    !esta en el lado derecho   
   vecinos_aux(4)=sitio + 1 - L  
  else   
   vecinos_aux(4)=sitio + 1  
  end if 
  
  vecinos=vecinos_aux  
 end function prox_vecinos 
 
 subroutine random_position(pos, n_spines)         !cada vez que llamemos a esta subrutina obtendremos la posicion de un spin aleatoriamente
  integer, intent(in):: n_spines                       !metemos el numero total de spines
  integer, intent(inout):: pos
  real:: aux
  real:: d         !otra auxiliar
  
  call random_seed()
  call random_number(aux)  
  d=aux*real(n_spines) + 1.0    
  pos=int(d)            !asi, obtenemos la misma probabilidad para los valores desde 1 hasta n_spines  
 end subroutine random_position
 
 subroutine conf_inicial(spines, n_spines, magn_inicial)      !magn_inicial es la magnetizacion que queremos que haya en la configuracion inicial  
  real, intent (in):: magn_inicial
  integer, intent (in):: n_spines
  integer, dimension(:), intent (inout):: spines
  integer:: posicion, contador
  integer:: n_sies, n_noes
  
  n_sies=nint((1.0 + magn_inicial)*real(n_spines)/2.0)      !obtenemos el numero de spines up que necesitamos para tener esa magn_inicial
  print *, "Numero de sies, para ver si es corrrecto: ", n_sies
    
  spines=0
  contador=0     !lo usaremos para saber a cuantos spines hemos asignado un 1(-1), parando el loop cuando contador=n_sies(n_noes)
  
  if (n_sies<=(n_spines/2)) then   
   do    
    if (contador==n_sies) then      !ya hemos asignado la cantidad que queriamos
     exit
    else if (contador>n_spines) then       !solo podemos asignar valores como mucho a n_spines spines
     print *, "Ha habido un error en la subrutina conf_inicial"
     stop 
    end if
    
    call random_position(posicion, n_spines)
    if (spines(posicion)==1) then         !ya hemos asignado un valor a ese spin
     cycle
    end if
    
    spines(posicion)=1
    contador=contador + 1    
   end do
  
   where (spines==0)    !asignamos -1 al resto de spines
    spines=-1
   end where   
  else   
   n_noes=n_spines - n_sies
   do   
    if (contador==n_noes) then      !ya hemos asignado la cantidad que queriamos
     exit
    else if (contador>n_spines) then       !solo podemos asignar valores como mucho a n_spines spines
     print *, "Ha habido un error en la subrutina conf_inicial"
     stop 
    end if
    
    call random_position(posicion, n_spines)
    if (spines(posicion)==-1) then         !ya hemos asignado un valor a ese spin
     cycle
    end if
    
    spines(posicion)=-1
    contador=contador + 1   
   end do
   
   where (spines==0)    !asignamos 1 al resto de spines
    spines=1
   end where   
  end if    
 end subroutine conf_inicial
  
 subroutine algoritmo(magn_deseada, n_spines, iteraciones_max, lista_vecinos)
  integer, intent (in):: n_spines
  real, intent (in):: magn_deseada
  integer, dimension (n_spines):: s, s_copia      !el indice del vector ira de 1 hasta L**2. Es el vector que contiene todos los spines
                                                 !en la copia guardaremos una configuracion inicial, la que usaremos en todas las simulaciones
  integer, dimension (:,:), intent (in):: lista_vecinos         !aqui estan los vecinos correspondientes a cada spin
  integer, intent (in):: iteraciones_max          !como mucho estas iteraciones para termalizar el sistema          
  integer:: i, t           !para hacer el loop de los procesos de montecarlo y el de los vecinos
  integer:: posicion1, posicion2             !para coger spines aleatoriamente, usando la subrutina del contains
  integer, dimension(4):: vecinos1, vecinos2         !para guardar las posiciones de los cuatro vecinos de cada uno de los dos spines
  real:: magn       !para ir calculando la magnetizacion actual
  integer:: aux         !a la hora de coger un vecino al azar de un spin, uso call random_position(aux, 4). Esto me devuelve en aux
                        !la posicion en vecinos1 que debo coger para la posicion en el vector de spines del spin vecino

  integer:: iterador, simulacion, contador_sies, contador_noes, contador_excepciones
  
  !para hacer los errores y medias
  real, dimension(1:10):: magn_media           !calcularemos la magnetizacion de la situacion final promediando todas las simulaciones 
  real:: valor_medio, error
    
  do iterador=1,10       !para conseguir 10 puntos para hacer las medias y error  
   !lo primero es conseguir la distribucion de spines inicial, con la subrutina del contains   
   call conf_inicial(s_copia,n_spines,magn_deseada)
       
   !ahora hay que hacer el algoritmo
   !simplemente cogemos un spin al azar y uno de sus vecinos
   !entonces, si opinan igual convencen a todos sus vecinos, si opinan diferente no ocurre nada
   !vamos a hacer varias simulaciones, y vamos a ver la media de la magnetizacion final, contando el n de veces que se llega a cada situacion
   
   contador_excepciones=0
   contador_sies=0
   contador_noes=0
   
   do simulacion=1,1000    
    if (modulo(simulacion, 250)==0) then            !para controlar la ejecucion del programa
     print *, "Va por la simulacion ", simulacion, " de la iteracion ", iterador
    end if
    
    s=s_copia                !inicializamos a la conf inicial que tenemos guardada
       
    do i=1,iteraciones     
     call random_position(posicion1, n_spines)      !generamos una posicion aleatoria de un spin
     vecinos1=lista_vecinos(posicion1,:)            !guardamos sus vecinos
     
     !elegimos uno de sus vecinos al azar (o sea, del 1 al 4 en el vector vecinos1, por lo que podemos usar la misma subrutina)
     call random_position(aux, 4)
     posicion2=vecinos1(aux)
     
     if (s(posicion1)*s(posicion2)==1) then       !tienen la misma opinion, por lo que se la imponen a sus vecinos. Si no, no hacemos nada      
      vecinos2=lista_vecinos(posicion2,:)      
      do t=1,4       
       s(vecinos1(t))=s(posicion1)
       s(vecinos2(t))=s(posicion1)       
      end do         
     end if 
     
     if (modulo(i,100)==0) then    !comprobamos si se ha alcanzado una de las dos situaciones finales cada 100 iteraciones      
      magn=real(sum(s))/real(n_spines)      
      if (magn==1.0 .or. magn==-1.0) then                 
       if (magn==1.0) then
        contador_sies=contador_sies + 1
       else 
        contador_noes=contador_noes + 1 
       end if       
       exit       
      end if      
     end if
     
     if (i==iteraciones) then            !puede que, incluso despues de todas las iteraciones, no se llegue a una de las situaciones
      contador_excepciones=contador_excepciones + 1      
     end if     
    end do
   end do
      
   magn_media(iterador)=real(contador_sies - contador_noes)/real(contador_sies + contador_noes)       
  end do
  
  valor_medio=sum(magn_media)/10.0
  
  error=0.0
  do i=1,10
   error=error + (magn_media(i) - valor_medio)**2.0
  end do
  error=sqrt(error/(10.0*9.0))
  
  write(unit=1, fmt="(f5.2,tr1,f7.4,tr1,f7.5)") magn_deseada, valor_medio, error  
 end subroutine algoritmo 
end program sznajd